@@ -29,6 +29,8 @@ use POSIX;
 use FileHandle;
 use Filesys::Df qw();
 use Time::Local;
+use IO::Zlib;
+use MIME::Base64;
 
 BEGIN {
     use Exporter ();
@@ -40,7 +42,7 @@ BEGIN {
 		 version_eq version_compare split_version
 		 binNMU_version parse_date isin copy dump_file
 		 check_packages help_text version_text usage_error
-		 send_mail debug df);
+		 send_mail send_build_log debug debug2 df);
 }
 
 our $devnull;
@@ -72,6 +74,7 @@ sub help_text ($$);
 sub version_text ($);
 sub usage_error ($$);
 sub debug (@);
+sub debug2 (@);
 
 sub version_less ($$) {
 	my $v1 = shift;
@@ -402,7 +405,8 @@ sub version_text ($) {
 $program (Debian sbuild) $Sbuild::Sysconfig::version ($Sbuild::Sysconfig::release_date)
 
 Written by Roman Hodek, James Troup, Ben Collins, Ryan Murray, Rick
-Younie, Francesco Paolo Lovergine, Michael Banck, and Roger Leigh
+Younie, Francesco Paolo Lovergine, Michael Banck, Roger Leigh and
+Andres Mejia.
 
 Copyright © 1998-2000 Roman Hodek <roman\@hodek.net>
           © 1998-1999 James Troup <troup\@debian.org>
@@ -410,7 +414,8 @@ Copyright © 1998-2000 Roman Hodek <roman\@hodek.net>
 	  © 2001-2003 Rick Younie <younie\@debian.org>
 	  © 2003-2004 Francesco Paolo Lovergine <frankie\@debian.org>
 	  © 2005      Michael Banck <mbanck\@debian.org>
-	  © 2005-2008 Roger Leigh <rleigh\@debian.org>
+	  © 2005-2010 Roger Leigh <rleigh\@debian.org>
+	  © 2009-2010 Andres Mejia <mcitadel\@gmail.com>
 
 This is free software; see the source for copying conditions.  There is NO
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -428,7 +433,73 @@ sub usage_error ($$) {
     exit 1;
 }
 
-sub send_mail {
+sub send_build_log ($$$$) {
+    my $conf = shift;
+    my $to = shift;
+    my $subject = shift;
+    my $file = shift;
+
+    # If no build log compression is desired, just pass this mail to the
+    # ordinary mailing function that also handles the other notifications.
+    if (!$conf->get('COMPRESS_BUILD_LOG_MAILS')) {
+        return send_mail($conf, $to, $subject, $file);
+    }
+
+    # This writes the compressed build log to yet another temporary file,
+    # generates base64 from it and pipes it into the mailer with
+    # Content-Type: application/x-gzip and Content-Transfer-Encoding:
+    # base64.
+    local( *F, *GZFILE );
+
+    if (!open( F, "<$file" )) {
+	warn "Cannot open $file for mailing: $!\n";
+	return 0;
+    }
+
+    my $tmp = File::Temp->new();
+    tie *GZFILE, 'IO::Zlib', $tmp->filename, 'wb';
+
+    while( <F> ) {
+        print GZFILE $_;
+    }
+    untie *GZFILE;
+
+    my $filename = $tmp->filename;
+    if (!open( F, "<$filename" )) {
+        warn "Cannot open $filename for mailing: $!\n";
+        return 0;
+    }
+
+    local $SIG{'PIPE'} = 'IGNORE';
+
+    if (!open( MAIL, "|" . $conf->get('MAILPROG') . " -oem $to" )) {
+	warn "Could not open pipe to " . $conf->get('MAILPROG') . ": $!\n";
+	close( F );
+	return 0;
+    }
+
+    print MAIL "From: " . $conf->get('MAILFROM') . "\n";
+    print MAIL "To: $to\n";
+    print MAIL "Subject: $subject\n";
+    print MAIL "Content-Type: application/x-gzip\n";
+    print MAIL "Content-Transfer-Encoding: base64\n";
+    print MAIL "\n";
+
+    my $buf;
+    while (read(F, $buf, 60*57)) {
+	print MAIL encode_base64($buf);
+    }
+
+    close( F );
+    if (!close( MAIL )) {
+	warn $conf->get('MAILPROG') . " failed (exit status $?)\n";
+	return 0;
+    }
+    return 1;
+}
+
+
+sub send_mail ($$$$) {
     my $conf = shift;
     my $to = shift;
     my $subject = shift;
@@ -475,6 +546,14 @@ sub debug (@) {
     }
 }
 
+sub debug2 (@) {
+
+    # TODO: Add debug level checking.
+    if ($debug_level && $debug_level >= 2) {
+	print STDERR "D2: ", @_;
+    }
+}
+
 sub df {
     my $dir = shift;
 
