@@ -44,9 +44,11 @@ use Module::Load::Conditional qw(can_load); # Used to check for LWP::UserAgent
 use Time::HiRes qw ( time ); # Needed for high resolution timers
 
 sub get_dist ($);
-sub setup ($$);
+sub setup ($$$);
 sub cleanup ($);
 sub shutdown ($);
+sub parse_file ($);
+sub dsc_files ($);
 
 my $current_session;
 
@@ -56,7 +58,7 @@ BEGIN {
 
     @ISA = qw(Exporter);
 
-    @EXPORT = qw(setup cleanup shutdown check_url download parse_file);
+    @EXPORT = qw(setup cleanup shutdown check_url download parse_file dsc_files);
 
     $SIG{'INT'} = \&shutdown;
     $SIG{'TERM'} = \&shutdown;
@@ -76,11 +78,11 @@ sub get_dist ($) {
     return $dist;
 }
 
-sub setup ($$) {
+sub setup ($$$) {
+    my $namespace = shift;
     my $chroot = shift;
     my $conf = shift;
 
-
     $conf->set('VERBOSE', 1);
     $conf->set('NOLOG', 1);
 
@@ -96,7 +98,8 @@ sub setup ($$) {
 
     my $session;
 
-    $session = $chroot_info->create($chroot,
+    $session = $chroot_info->create($namespace,
+				    $chroot,
 				    undef, # TODO: Add --chroot option
 				    $conf->get('ARCH'));
 
@@ -358,7 +361,7 @@ sub _get_proxy {
 # It can also be used on files like Packages or Sources files in a Debian
 # archive.
 # This subroutine returns an array of hashes. Each hash is a stanza.
-sub parse_file {
+sub parse_file ($) {
     # Takes one parameter, the file to parse.
     my ($file) = @_;
 
@@ -388,7 +391,11 @@ sub parse_file {
     {
         # Attempt to open and read the file
         my $fh;
-        open $fh, '<', $file or die "Could not read $file: $!";
+        if (ref($file) eq "GLOB") {
+          $fh = $file;
+        } else {
+          open $fh, '<', $file or die "Could not read $file: $!";
+        }
 
         # Read paragraph by paragraph
         local $/ = "";
@@ -407,6 +414,10 @@ sub parse_file {
                 my ($field, $field_contents);
                 $field = $1 if ($match =~ /([^:]+?):/msx);
                 $field_contents = $1 if ($match =~ /[^:]+?:(.*)/msx);
+
+                # Trim leading and trailing space from entries
+                $field_contents =~ s/^[\s]+?\b//msx;
+                $field_contents =~ s/[\s]+?$//msx;
                 $fields{$field} = $field_contents;
             }
 
@@ -420,4 +431,26 @@ sub parse_file {
     return \@array_of_fields;
 }
 
+sub dsc_files ($) {
+    my $dsc = shift;
+
+    my @files;
+
+    # The parse_file() subroutine returns a ref to an array of hashrefs.
+    my $stanzas = parse_file($dsc);
+
+    # A dsc file would only ever contain one stanza, so we only deal with
+    # the first entry which is a ref to a hash of fields for the stanza.
+    my $stanza = @{$stanzas}[0];
+
+    # We're only interested in the name of the files in the Files field.
+    my $entry = ${$stanza}{'Files'};
+
+    foreach my $line (split("\n", $entry)) {
+	push @files, $1 if $line =~ /(\S+)\s*$/;
+    }
+
+    return @files;
+}
+
 1;
